# Copyright (C) 2019 - 2020 Satoru SATOH <satoru.satoh@gmail.com>
# SPDX-License-Identifier: MIT
#
# Ref. python -c "import json; help(json)"
#
"""JSON5 Backend.

- Format to support: JSON5, https://json5.org/
- Requirements: pyjson5
- Development Status :: 3 - Alpha
- Limitations: None obvious

Changelog:

.. versionadded:: 0.0.1
"""
import json5
import anyconfig.backend.base


_LOAD_OPTS = """object_hook parse_float parse_int
parse_constant object_pairs_hook""".split()

_DUMP_OPTS = """skipkeys ensure_ascii check_circular allow_nan
indent separators default sort_keys""".split()

_DICT_OPTS = 'object_pairs_hook object_hook'.split()


class Parser(anyconfig.backend.base.StringStreamFnParser):
    """
    Parser for JSON5 data files.
    """
    _cid = 'json5'
    _type = 'json5'
    _ordered = True
    _allow_primitives = True

    _load_opts = _LOAD_OPTS
    _dump_opts = _DUMP_OPTS
    _dict_opts = _DICT_OPTS

    _load_from_string_fn = anyconfig.backend.base.to_method(json5.loads)
    _load_from_stream_fn = anyconfig.backend.base.to_method(json5.load)
    _dump_to_string_fn = anyconfig.backend.base.to_method(json5.dumps)
    _dump_to_stream_fn = anyconfig.backend.base.to_method(json5.dump)

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2021 Satoru SATOH <satoru.satoh@gmail.com>
# SPDX-License-Identifier: MIT
#
# pylint: disable=missing-docstring,invalid-name
import unittest
import anyconfig
from . import CNF_FILES


class Test_90(unittest.TestCase):

    def _try_loads(self, files):
        for filepath in files:
            try:
                cnf = anyconfig.load(filepath, ac_parser='json5')
                self.assertTrue(cnf)

                exp_path = filepath.parent / filepath.name.replace('.json5',
                                                                   '.json')
                ref = anyconfig.load(exp_path, ordered=True)

            except anyconfig.UnknownFileTypeError:
                print('all types=%r' % anyconfig.list_types())
                raise

            self.assertEqual(cnf, ref)

    def test_10_load(self):
        self._try_loads(CNF_FILES)

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2021 Satoru SATOH <satoru.satoh@gmail.com>
# SPDX-License-Identifier: MIT
#
# pylint: disable=missing-docstring,invalid-name,too-few-public-methods
import collections
import pathlib
import tempfile
import unittest

import anyconfig
import anyconfig.ioinfo
import anyconfig_json5_backend.json5_ as TT

from . import CNF_FILES


class TestCase(unittest.TestCase):

    maxDiff = None

    def test_load(self):
        self.assertTrue(CNF_FILES)

        for in_path in CNF_FILES:
            exp_path = in_path.parent / in_path.name.replace('.json5', '.json')

            with tempfile.TemporaryDirectory() as tmp_path:
                out_path = pathlib.Path(tmp_path) / 'out.json'

                if not exp_path.exists():
                    continue  # The reference test data is not ready.

                try:
                    cnf = TT.Parser().load(anyconfig.ioinfo.make(in_path),
                                           ac_ordered=True)
                    self.assertTrue(cnf)
                    self.assertTrue(isinstance(cnf, collections.OrderedDict))

                    anyconfig.dump(cnf, out_path)
                    self.assertEqual(anyconfig.load(out_path,
                                                    ac_ordered=False),
                                     anyconfig.load(exp_path,
                                                    ac_ordered=False))
                except AssertionError as exc:
                    raise AssertionError("file: {}, "
                                         "exc={!s}".format(in_path, exc))

# vim:sw=4:ts=4:et:
